/*
	projShadows.frag - fragment shader for shadow mapping
	
	texture_map - standard u,v map for object you're drawing
	glow_map	- map that specifies amount of glow at each texture location
	depth_map	- depth buffer from rendering scene from light's perspective
	
	use_glow	- are we actually using a glow map?
	use_shadow	- enable shadowing?
	
	diffuse, ambient, lightDir, halfVector, normal
		- standard lighting variables passed from vertex shader for per pixel lighting

	The lighting part of this shader was lifted from LightHouse3D's tutorial
	http://www.lighthouse3d.com/opengl/glsl/index.php?dirlightpix
	
	It's specifically the shadow mapping stuff that is relevent for this shader

*/

uniform sampler2D texture_map;
uniform sampler2DShadow depth_map;

uniform float shadowSize;
uniform float shadowOpacity;

varying vec4 diffuse, ambient;
varying vec3 lightDir, halfVector, normal;

float pcflookup(vec3 loc) {
	
	float upp = 1.0 / shadowSize;
	
	vec2 bias = fract(loc.xy * shadowSize);
	loc -= vec3(upp * bias, 0.0);
	
	float LD = shadow2D(depth_map, loc).r;
	float RD = shadow2D(depth_map, loc + upp * vec3(1.0, 0.0, 0.0)).r;
	float LU = shadow2D(depth_map, loc + upp * vec3(0.0, 1.0, 0.0)).r;
	float RU = shadow2D(depth_map, loc + upp * vec3(1.0, 1.0, 0.0)).r;
	
	float up = bias.y;
	float down = 1.0 - bias.y;
	float right = bias.x;
	float left = 1.0 - bias.x;
	
	return left * ( up * LU + down * LD) + right * ( up * RU + down * RD);
	
}

void main() {

	vec4 samp   = texture2D(texture_map, gl_TexCoord[0].xy);
	
	float shadow = shadowOpacity * pcflookup(gl_TexCoord[1].xyz) + (1.0 - shadowOpacity);
	//float shadow = shadow2D(depth_map, gl_TexCoord[1].xyz).r;

	vec3 n, halfV;
	float NdotL, NdotHV;
	
	vec4 color = ambient;
	n = normalize(normal);
	
	NdotL = max(dot(n, lightDir), 0.0);
	
	color += diffuse * NdotL * shadow;
	halfV = normalize(halfVector);
	NdotHV = max(dot(n, halfV), 0.0);
	color += gl_FrontMaterial.specular * gl_LightSource[0].specular * pow(NdotHV, gl_FrontMaterial.shininess) * shadow;
	
	gl_FragColor = color * samp;
	
}